/*
 * Soubor:  main.c
 * Datum:   01.08.2004, 23.8.2023
 * Autor:   David Martinek, david.martinek.78 (zavináč) gmail.com
 * Projekt: Datumy, vzorový projekt
 * Popis:   Program provádí výpočet rozdílu mezi dvěmi kalendářními daty.
 *
 * Zdrojový text je v kódování UTF-8. Při použití ve Windows je potřeba jej
 * překódovat do CP852. V Linuxu (např. v replit.com) český výstup v UTF-8
 * funguje bez problémů.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <stdbool.h>
#include <limits.h>


const unsigned int MAX_YEAR = (unsigned int)(UINT_MAX / 365.25);
// #define MAX_YEAR 11000000

/**
 * Vytiskne na standardní výstup text s nápovědou.
 */
void printHelp()
{
  printf("Program Datumy - vzorová implementace.\n"
         "Autor: David Martinek (c) 2004--2023\n"
         "Program počítá rozdíl zadaných kalendářních dat.\n"
         "Data jsou načítána ze standardního vstupu ve formátu\n"
         "dd.mm.rrrr-dd.mm.rrrr\n"
         "Rozsah povolených let je 1 - %d\n"
         "Popis parametrů:\n"
         "  -h  Vypíše tuto obrazovku s nápovědou.\n", MAX_YEAR);
}

/**
 * Zpracuje argumenty příkazového řádku. Pokud narazí na argument -h, vypíše
 * nápovědu a ukončí program.
 * @param argc Počet argumentů.
 * @param argv Pole textových řetězců s argumenty.
 */
void doParams(int argc, char *argv[])
{
  if (argc == 2 && strcmp("-h", argv[1]) == 0)
  { // tisk nápovědy
    printHelp();
    exit(EXIT_SUCCESS); // s tímto opatrně!
  }
}

/**
 * Struktura pro reprezentaci jediného data.
 */
typedef struct tdate
{
  unsigned int day;
  unsigned int month;
  unsigned int year;
} TDate;

/**
 * Vrací true, pokud je parametr přestupný rok.
 */
bool isLeapYear(unsigned int year)
{
  return (year % 4 == 0) &&
         ((year % 100 != 0) || (year % 400 == 0));
}

/**
 * Vrací true, pokud je hodnota datumu v pořádku.
 */
bool isDateOk(const TDate *date)
{
  bool ok = date->month > 0 && date->month <= 12 &&
            date->year > 0 && date->year <= MAX_YEAR;
  if (!ok) return false;

  unsigned int maxDay = 31;
  if (isLeapYear(date->year))
  {
    if (date->month == 2) maxDay = 29;
  }
  else
  {
    if (date->month == 2) maxDay = 28;
  }

  if (date->month == 4 || date->month == 6 ||
      date->month == 9 || date->month == 11)
  {
    maxDay = 30;
  }

  ok = date->day > 0 && date->day <= maxDay;
  return ok;
  // Poznámka: Normálně nejsem zastáncem "magických konstant", ale v této funkci
  // jde o natolik běžně používané hodnoty, které se navíc nikdy nebudou měnit
  // (leda by při dalším převratu někdo přečísloval měsíce), že by tady pojmenované
  // konstanty nepřinesly žádnou zásadní výhodu. Jinak ale platí, že
}

/**
 * Vrací počet dnů od začátku letopočtu do začátku zadaného roku (přičítá i rok
 * "0", ale potom se počítá rozdíl, takže to nehraje roli).
 */
unsigned int getDaysToYear(unsigned int year)
{
  year -= 1;  //aktuální rok nechci započítat, ten se započítá jinde
  return year*365 + year/4 - year/100 + year/400;
}

/**
 * Vrátí počet dnů od začátku letopočtu do zadaného data.
 */
unsigned int getDays(const TDate *date)
{
  static const unsigned int daysToMonth[12] =
  {
    0,
    31,
    31+28,
    31+28+31,
    31+28+31+30,
    31+28+31+30+31,
    31+28+31+30+31+30,
    31+28+31+30+31+30+31,
    31+28+31+30+31+30+31+31,
    31+28+31+30+31+30+31+31+30,
    31+28+31+30+31+30+31+31+30+31,
    31+28+31+30+31+30+31+31+30+31+30
  };
  unsigned int days = getDaysToYear(date->year) + daysToMonth[date->month-1]
    + date->day;

  if (isLeapYear(date->year) && date->month > 2)
  {
    days += 1;
  }

  return days;
}

/** Kódy chyb, které se mohou vyskytnout ve funkci readDates. */
enum terrors
{
  EOK,      /**< Bez chyby. */
  EFORMAT,  /**< Chyba ve formátu vstupního výrazu. */
  EDATE1,   /**< Nesmyslná hodnota prvního datumu. */
  EDATE2,   /**< Nesmyslná hodnota druhého datumu. */
  EUNKNOWN  /**< Neznámá chyba. */
};

/**
 * Chybová hlášení. Jejich pořadí v poli odpovídá pořadí konstant
 * ve výčtu terrors.
 */
const char *EMSG[] =
{
  "Vše v pořádku.",
  "Chybný formát vstupního výrazu.",
  "První datum má nesmyslnou hodnotu.",
  "Druhé datum má nesmyslnou hodnotu.",
  "Neznámá chyba! Něco je tragicky špatně!"
};

/**
 * Vytiskne chybové hlášení na chybový výstup.
 */
void printError(int error)
{
  if (error < EOK || error >= EUNKNOWN)
    error = EUNKNOWN;
  fprintf(stderr, "Chyba: %s\n", EMSG[error]);
}


/**
 * Přečte výraz ve vstupním streamu a převede jej na dva datumy. Pokud
 * je výraz zadán chybně, vrátí chybový kód.
 * @param stream Vstupní stream (soubor nebo stdin) ze kterého se bude číst.
 * @param date1 První datum nalezené v textovém řetězci. (Předávání odkazem.)
 * @param date2 Druhé datum nalezené v textovém řetězci. (Předávání odkazem.)
 * @return Vrací chybový kód z výčtu enum terrors.
 */
int readDates(FILE *stream, TDate *date1, TDate *date2)
{
  assert(stream != NULL && date1 != NULL && date2 != NULL);

  int num = fscanf(stream, "%u.%u.%u-%u.%u.%u",
      &date1->day, &date1->month, &date1->year,
      &date2->day, &date2->month, &date2->year);

  if (num != 6) return EFORMAT; // očekávám, že fscanf zpracovala 6 položek
  if (!isDateOk(date1)) return EDATE1;
  if (!isDateOk(date2)) return EDATE2;
  return EOK;
}

/**
 * Odečte dvě data a vrátí počet dnů mezi těmito datumy.
 */
unsigned int getDiff(const TDate *date1, const TDate *date2)
{
  unsigned int d1 = getDays(date1);
  unsigned int d2 = getDays(date2);
  return (d1 < d2)? d2 - d1 : d1 - d2;
}

/////////////////////////////////////////////////////
/**
 * Hlavní program.
 */
int main(int argc, char *argv[])
{
  doParams(argc, argv);

  TDate date1, date2;
  int err;

  if ((err = readDates(stdin, &date1, &date2)) != EOK)
  {
    fprintf(stderr, "Chyba při načítání vstupních dat!\n");
    printError(err);
    return EXIT_FAILURE;
  }

//  printf("Datum1: %d.%d.%d\n", date1.day, date1.month, date1.year);
//  printf("Datum2: %d.%d.%d\n", date2.day, date2.month, date2.year);
//  printf("Rozdíl: %u\n", getDiff(date1, date2));
  printf("%u\n", getDiff(&date1, &date2));

  return EXIT_SUCCESS;
}

/* konec souboru main.c */
